let Genode = env:DHALL_GENODE

let Init = Genode.Init

let Child = Init.Child

let ServiceRoute = Init.ServiceRoute

let routeLogRom =
      λ(label : Text) → ServiceRoute.parentLabel "ROM" (Some "log") (Some label)

in  Init::{
    , verbose = True
    , children = toMap
        { fs_log =
            Child.flat
              Child.Attributes::{
              , binary = "fs_log"
              , config = Init.Config::{
                , defaultPolicy = Some Init.Config.DefaultPolicy::{
                  , attributes = toMap { merge = "yes", truncate = "yes" }
                  }
                }
              , exitPropagate = True
              , provides = [ "LOG" ]
              , routes = [ ServiceRoute.parent "File_system" ]
              }
        , log_core =
            Child.flat
              Child.Attributes::{
              , binary = "log_core"
              , routes =
                [ routeLogRom "core_log"
                , ServiceRoute.childLabel
                    "LOG"
                    "fs_log"
                    (Some "log")
                    (Some "core")
                ]
              }
        , log_kernel =
            Child.flat
              Child.Attributes::{
              , binary = "log_core"
              , routes =
                [ routeLogRom "kernel_log"
                , ServiceRoute.childLabel
                    "LOG"
                    "fs_log"
                    (Some "log")
                    (Some "kernel")
                ]
              }
        }
    , routes = [ ServiceRoute.parent "Timer" ]
    }
