# This file contains overrides necessary to build some Make and Depot targets.
# Many targets can be built with the default attributes, and are not listed here.
# However, any package listed here with empty overrides ({ }) will be added to
# the package attributes of this flake.

{ buildPackages, genodePackages, ports }:
with ports;
let
  self = genodePackages;

  includeDir = pkg: buildPackages.lib.getDev pkg + "/include";

  hostLibcInc = includeDir buildPackages.glibc;
  # TODO: does this need to be glibc?

  vbox5' = {
    nativeBuildInputs = with buildPackages; [ iasl yasm ];
    portInputs = [ libc libiconv qemu-usb stdcxx virtualbox5 ];
  };
in {

  acpi_drv = { };

  cached_fs_rom.patches = [ ./cached_fs_rom.patch ];

  fb_sdl = with buildPackages; {
    nativeBuildInputs = [ pkgconfig ];
    buildInputs = [ SDL ];
    HOST_INC_DIR = [ hostLibcInc (includeDir SDL) ];
  };

  gpt_write.portInputs = [ jitterentropy ];

  init.patches = [ ./sandbox.patch ./xml-fail.patch ];

  event_filter.patches = [ ./event_filter.patch ];

  intel_fb_drv = {
    BOARD = "pc";
    portInputs = [ dde_linux ];
  };

  ipxe_nic_drv.portInputs = [ dde_ipxe ];

  libc = {
    buildInputs = [ self.vfs ];
    portInputs = [ libc ];
    patches = [ ./libc.patch ];
  };

  libiconv.portInputs = [ libc libiconv ];

  linux_nic_drv.HOST_INC_DIR = [ hostLibcInc ];

  lx_block.HOST_INC_DIR = [ hostLibcInc ];

  nic_bridge = { };

  nic_loopback = { };

  noux.portInputs = [ libc ];

  platform_drv = { };

  posix = {
    buildInputs = [ self.libc ];
    portInputs = [ ports.libc ];
  };

  report_rom = { };

  rom_logger = { };

  rtc_drv.meta.platforms = [ "x86_64-genode" ];

  rump = {
    portInputs = [ dde_rump ];
    buildInputs = with buildPackages; [ zlib ];
  };

  sequence = { };

  stdcxx.portInputs = [ libc stdcxx ];

  # The following are tests are patched to exit at completion

  "test-log".patches = [ ./test-log.patch ];

  "test-signal".patches = [ ./test-signal.patch ];

  usb_drv.portInputs = [ dde_linux ];

  vbox5 = vbox5';

  vbox5-nova = vbox5';

  vesa_drv.portInputs = [ libc x86emu ];

  vfs.outputs = [ "out" "lib" ];
  vfs_audit = { };
  vfs_block = { };
  vfs_import.patches = [ ./vfs_import.patch ];
  vfs_jitterentropy.portInputs = [ jitterentropy libc ];
  vfs_lwip.portInputs = [ lwip ];
  vfs_pipe = { };
  vfs_ttf.portInputs = [ libc stb ];

  virtdev_rom = { };
  virtio_nic_drv = { };

  wifi_drv.portInputs = [ dde_linux libc openssl ];

}
